from klefki.types.algebra.abstract import Group, Field
from functools import reduce
from typing import Iterator
from operator import add

__all__ = ['commitment', 'vertex_commitment', 'com']

def v_multi(g: [Group], a: [Field]) -> [Group]:
    return reduce(lambda x,y: x+y,
                  list(map(lambda a: a[0] @ a[1], zip(g, a))))


def commitment(x: Field, r: Field, H: Group, G: Group) -> Group:
    return (G ^ x) * (H ^ r)


def vertex_commitment(x: [Field], r: Field, H: Group, G: [Group]) -> Group:
    return v_multi(G, x) * H ^ r


def matrix_commitment(x: [[Field]], r: Field, H: Group, G: [Group]) -> Group:
    return vertex_commitment(G, H, reduce(add, x), r)

def com(x, r, H, G) -> Group:
    if type(x) in [
            Iterator,
            list
    ]:
        if type(x[0]) in [
            Iterator,
            list
        ]:
            return matrix_commitment(x, r, H, G)
        return vertex_commitment(x, r, H, G)
    return commitment(x, r, H, G)



from math import gcd


def carmichael(n):
'''
A number n is said to be a Carmichael number if it satisfies the following modular arithmetic condition:

 power(b, n-1) MOD n = 1,
  for all b ranging from 1 to n such that b and
  n are relatively prime, i.e, gcd(b, n) = 1
'''

    coprimes = [x for x in range(1, n) if gcd(x, n) == 1]
    k = 1
    while not all(pow(x, k, n) == 1 for x in coprimes):
        k += 1
    return k



def lcm(a, b):
    return abs(a*b) // gcd(a, b)
